@@ -78,7 +78,7 @@ typedef struct _CtkDisplayConfig
 {
     GtkVBox parent;
 
-    NvCtrlAttributeHandle *handle;
+    CtrlTarget *ctrl_target;
     CtkConfig *ctk_config;
 
     gboolean page_selected; /* Is the display config page selected in the UI */
@@ -93,6 +93,9 @@ typedef struct _CtkDisplayConfig
     GtkWidget *chk_primary_display;
     gboolean primary_display_changed;
 
+    GtkWidget *chk_forcecompositionpipeline_enabled;
+    GtkWidget *chk_forcefullcompositionpipeline_enabled;
+
     GtkWidget *mnu_selected_item;
     SelectableItem *selected_item_table;
     int selected_item_table_len;
@@ -136,7 +139,7 @@ typedef struct _CtkDisplayConfig
     GtkWidget *box_display_underscan;
     GtkWidget *txt_display_underscan;
     GtkWidget *sld_display_underscan;
-    GtkObject *adj_display_underscan;
+    GtkAdjustment *adj_display_underscan;
 
     GtkWidget *box_display_viewport;
 
@@ -243,8 +246,7 @@ typedef struct _CtkDisplayConfigClass
 
 
 GType       ctk_display_config_get_type  (void) G_GNUC_CONST;
-GtkWidget*  ctk_display_config_new       (NvCtrlAttributeHandle *,
-                                          CtkConfig *);
+GtkWidget*  ctk_display_config_new       (CtrlTarget *, CtkConfig *);
 
 GtkTextBuffer *ctk_display_config_create_help(GtkTextTagTable *,
                                               CtkDisplayConfig *);
